"""Helper file for making generated modules accessible."""
import os
import sys

# Adds this directory to the Python path. Then, the generated files can be found.
sys.path.append(os.path.abspath(os.path.dirname(__file__)))



"""
@generated by mypy-protobuf.  Do not edit manually!
isort:skip_file
"""
import builtins
import google.protobuf.descriptor
import google.protobuf.internal.containers
import google.protobuf.message
import typing
import typing_extensions

DESCRIPTOR: google.protobuf.descriptor.FileDescriptor

class ChangeVolume(google.protobuf.message.Message):
    DESCRIPTOR: google.protobuf.descriptor.Descriptor
    NAME_TOKENS_FIELD_NUMBER: builtins.int
    VALUE_FIELD_NUMBER: builtins.int
    RELATIVE_FIELD_NUMBER: builtins.int
    @property
    def name_tokens(self) -> google.protobuf.internal.containers.RepeatedScalarFieldContainer[typing.Text]: ...
    value: builtins.float
    relative: builtins.bool
    def __init__(self,
        *,
        name_tokens: typing.Optional[typing.Iterable[typing.Text]] = ...,
        value: builtins.float = ...,
        relative: builtins.bool = ...,
        ) -> None: ...
    def ClearField(self, field_name: typing_extensions.Literal["name_tokens",b"name_tokens","relative",b"relative","value",b"value"]) -> None: ...
global___ChangeVolume = ChangeVolume

class ControlParameterRequest(google.protobuf.message.Message):
    DESCRIPTOR: google.protobuf.descriptor.Descriptor
    CHANGE_VOLUME_FIELD_NUMBER: builtins.int
    @property
    def change_volume(self) -> global___ChangeVolume: ...
    def __init__(self,
        *,
        change_volume: typing.Optional[global___ChangeVolume] = ...,
        ) -> None: ...
    def HasField(self, field_name: typing_extensions.Literal["change_volume",b"change_volume","command",b"command"]) -> builtins.bool: ...
    def ClearField(self, field_name: typing_extensions.Literal["change_volume",b"change_volume","command",b"command"]) -> None: ...
    def WhichOneof(self, oneof_group: typing_extensions.Literal["command",b"command"]) -> typing.Optional[typing_extensions.Literal["change_volume"]]: ...
global___ControlParameterRequest = ControlParameterRequest

class LoadParameters(google.protobuf.message.Message):
    DESCRIPTOR: google.protobuf.descriptor.Descriptor
    CONTENT_FIELD_NUMBER: builtins.int
    @property
    def content(self) -> google.protobuf.internal.containers.RepeatedScalarFieldContainer[typing.Text]: ...
    def __init__(self,
        *,
        content: typing.Optional[typing.Iterable[typing.Text]] = ...,
        ) -> None: ...
    def ClearField(self, field_name: typing_extensions.Literal["content",b"content"]) -> None: ...
global___LoadParameters = LoadParameters

class ControlRequest(google.protobuf.message.Message):
    DESCRIPTOR: google.protobuf.descriptor.Descriptor
    RESET_DSP_FIELD_NUMBER: builtins.int
    HARD_RESET_DSP_FIELD_NUMBER: builtins.int
    LOAD_PARAMETERS_FIELD_NUMBER: builtins.int
    reset_dsp: builtins.bool
    hard_reset_dsp: builtins.bool
    @property
    def load_parameters(self) -> global___LoadParameters: ...
    def __init__(self,
        *,
        reset_dsp: builtins.bool = ...,
        hard_reset_dsp: builtins.bool = ...,
        load_parameters: typing.Optional[global___LoadParameters] = ...,
        ) -> None: ...
    def HasField(self, field_name: typing_extensions.Literal["command",b"command","hard_reset_dsp",b"hard_reset_dsp","load_parameters",b"load_parameters","reset_dsp",b"reset_dsp"]) -> builtins.bool: ...
    def ClearField(self, field_name: typing_extensions.Literal["command",b"command","hard_reset_dsp",b"hard_reset_dsp","load_parameters",b"load_parameters","reset_dsp",b"reset_dsp"]) -> None: ...
    def WhichOneof(self, oneof_group: typing_extensions.Literal["command",b"command"]) -> typing.Optional[typing_extensions.Literal["reset_dsp","hard_reset_dsp","load_parameters"]]: ...
global___ControlRequest = ControlRequest

class ControlResponse(google.protobuf.message.Message):
    DESCRIPTOR: google.protobuf.descriptor.Descriptor
    SUCCESS_FIELD_NUMBER: builtins.int
    MESSAGE_FIELD_NUMBER: builtins.int
    success: builtins.bool
    message: typing.Text
    def __init__(self,
        *,
        success: builtins.bool = ...,
        message: typing.Text = ...,
        ) -> None: ...
    def ClearField(self, field_name: typing_extensions.Literal["message",b"message","success",b"success"]) -> None: ...
global___ControlResponse = ControlResponse



"""The backend application of sigmadsp.

This module includes the main application that provides an interface between the TCP server that faces SigmaStudio,
and an SPI handler that controls a DSP.

Commands from Sigma Studio are received, and translated to SPI read/write requests.
"""
import argparse
import logging
import sched
import sys
import threading
import time
from concurrent.futures import ThreadPoolExecutor

import grpc
from retry import retry

import sigmadsp
from sigmadsp.communication.sigma_tcp_server import (
    ReadRequest,
    ReadResponse,
    SigmaStudioInterface,
    WriteRequest,
)
from sigmadsp.generated.backend_service.control_pb2 import (
    ControlParameterRequest,
    ControlRequest,
    ControlResponse,
)
from sigmadsp.generated.backend_service.control_pb2_grpc import (
    BackendServicer,
    add_BackendServicer_to_server,
)
from sigmadsp.hardware.adau14xx import Adau14xx
from sigmadsp.hardware.dsp import SafetyCheckException
from sigmadsp.hardware.spi import SpiHandler
from sigmadsp.helper.settings import SigmadspSettings

# A logger for this module
logger = logging.getLogger(__name__)


class BackendService(BackendServicer):
    """The backend service that handles the underlying TCP server and SPI handler.

    This service also reacts to rpyc remote procedure calls, for performing actions with the DSP over SPI.
    """

    def __init__(self, settings: SigmadspSettings):
        """Initialize service and start all relevant threads (TCP, SPI).

        Args:
            settings (SigmadspSettings): The settings object.
        """
        super().__init__()

        # If configuration_unlocked is False, no DSP parameters can be changed.
        self.configuration_unlocked: bool = False

        self.settings = settings

        # Create an SPI handler, along with its thread
        self.spi_handler = SpiHandler()

        # Create a SigmaTCPServer, along with its various threads
        self.sigma_tcp_server = SigmaStudioInterface(
            self.settings.config["host"]["ip"],
            self.settings.config["host"]["port"],
        )
        logger.info(
            "Sigma TCP server started on [%s]:%d.",
            self.settings.config["host"]["ip"],
            self.settings.config["host"]["port"],
        )

        # Create a scheduler for recurring tasks
        self.scheduler = sched.scheduler(time.time, time.sleep)

        # Create the worker thread for the handler itself
        worker_thread = threading.Thread(target=self.worker, name="Backend service worker thread")
        worker_thread.daemon = True
        worker_thread.start()

        logger.info("Specified DSP type is '%s'.", self.settings.config["dsp"]["type"])

        if self.settings.config["dsp"]["type"] == "adau14xx":
            self.dsp = Adau14xx(self.settings.config, self.spi_handler)

        else:
            logger.error(
                "DSP type '%s' is not known! Aborting.",
                self.settings.config["dsp"]["type"],
            )
            sys.exit(0)

        try:
            logger.info("Run startup safety check.")
            self.startup_safety_check()

        except SafetyCheckException:
            logger.warning("Startup safety check failed.")

    @retry(SafetyCheckException, 5, 5)
    def startup_safety_check(self) -> None:
        """Perform startup safety check, retrying a few times.

        The safety check might not be successful immediately after starting the DSP, so it is retried periodically.
        """
        self.safety_check()

    def safety_check(self) -> None:
        """Check a hash cell within the DSP's memory against a hash value from the parameter file.

        Only if they match, configuration of DSP parameters is allowed. Other operations (e.g. reset, or
        loading a new parameter file) are still allowed, even in a locked configuration state.
        """
        if not self.settings.parameter_parser:
            logger.warning("No parameter file was loaded! Configuration remains locked.")
            return

        safety_hash_cell = self.settings.parameter_parser.safety_hash_cell

        if safety_hash_cell is None:
            logger.warning("No safety hash cell exists in the DSP configuration! Configuration remains locked.")
            self.configuration_unlocked = False

        else:
            dsp_hash = self.dsp.get_parameter_value(safety_hash_cell.parameter_address, data_format="int")

            if safety_hash_cell.parameter_value != dsp_hash:
                logger.warning("Safety hash cell content does not match! Configuration remains locked.")
                self.configuration_unlocked = False
                raise SafetyCheckException

            else:
                logger.info("Safety check successful. Configuration unlocked.")
                self.configuration_unlocked = True

    def worker(self):
        """Main worker functionality.

        Gets requests from the TCP server component and forwards them to the SPI handler.
        """
        while True:
            request = self.sigma_tcp_server.pipe_end_user.recv()

            if isinstance(request, WriteRequest):
                self.spi_handler.write(request.address, request.data)

            elif isinstance(request, ReadRequest):
                payload = self.spi_handler.read(request.address, request.length)

                self.sigma_tcp_server.pipe_end_user.send(ReadResponse(payload))

    def control_parameter(self, request: ControlParameterRequest, context):
        """Main backend entry point for control messages that change or read parameters.

        Here, the configuration has to be in an unlocked state, which requires a hash number in the DSP firmware to
        match the hash number that is provided in the parameter file.

        Args:
            request (ControlParameterRequest): The request that the backend shall handle.
            context (Any): The context within which to handle the request (unused).

        Returns:
            ControlResponse: A control response message, returned to the caller.
        """
        response = ControlResponse()

        # Determine the type of control request.
        command = request.WhichOneof("command")
        response.success = False

        if not self.settings.parameter_parser:
            response.message = "No parameters loaded, parameters cannot be changed."
            return response

        if not self.configuration_unlocked:
            # Do not allow to change parameters.
            response.message = "Configuration locked, parameters cannot be changed."
            return response

        if "change_volume" == command:
            volume_cells_to_adjust = self.settings.parameter_parser.get_matching_cells_by_name_tokens(
                self.settings.parameter_parser.volume_cells, list(request.change_volume.name_tokens)
            )

            if not volume_cells_to_adjust:
                response.message = (
                    f"No volume cell identified by {request.change_volume.name_tokens} was found."
                    "Available controls are "
                    f"{([cell.name_tokens for cell in self.settings.parameter_parser.volume_cells])}."
                )

            for volume_cell in volume_cells_to_adjust:
                if request.change_volume.relative:
                    new_volume_db = self.dsp.adjust_volume(
                        request.change_volume.value,
                        volume_cell.parameter_address,
                    )

                elif not request.change_volume.relative:
                    new_volume_db = self.dsp.set_volume(
                        request.change_volume.value,
                        volume_cell.parameter_address,
                    )

                response.message = f"Set volume of cell '{volume_cell.full_name}' to {new_volume_db:.2f} dB."

        response.success = True
        return response

    def control(self, request: ControlRequest, context):
        """Main backend entry point for control messages.

        Args:
            request (ControlRequest): The request that the backend shall handle.
            context (Any): The context within which to handle the request (unused).

        Returns:
            ControlResponse: A control response message, returned to the caller.
        """
        response = ControlResponse()

        # Determine the type of control request.
        command = request.WhichOneof("command")

        if "reset_dsp" == command:
            self.dsp.soft_reset()
            response.message = "Soft-reset DSP."
            response.success = True

        elif "hard_reset_dsp" == command:
            self.dsp.hard_reset()
            response.message = "Hard-reset DSP."
            response.success = True

        elif "load_parameters" == command:
            self.settings.store_parameters(list(request.load_parameters.content))

            # Repeat safety check after loading a new set of parameters
            try:
                self.safety_check()

            except SafetyCheckException:
                pass

            if self.configuration_unlocked:
                response.message = "Loaded parameters, control is unlocked."
                response.success = True

            else:
                response.message = "Safety check failed, parameters cannot be adjusted."
                response.success = False

        return response


def launch(settings: SigmadspSettings):
    """Launch the backend application.

    Args:
        settings (SigmadspSettings): Settings object for the backend application.
    """
    # Create the backend service, a grpc service
    grpc_server = grpc.server(ThreadPoolExecutor(max_workers=10))

    backend_port = settings.config["backend"]["port"]
    backend_address = f"[::]:{backend_port}"

    add_BackendServicer_to_server(BackendService(settings), grpc_server)
    grpc_server.add_insecure_port(backend_address)
    grpc_server.start()

    logger.info("Backend service started on %s", backend_address)

    grpc_server.wait_for_termination()


def main():
    """Launch the backend with default settings."""
    logging.basicConfig(level=logging.INFO)

    logger.info("Starting the sigmadsp backend, version %s.", sigmadsp.__version__)

    argument_parser = argparse.ArgumentParser()
    argument_parser.add_argument(
        "-s",
        "--settings",
        required=False,
        help=(
            "specifies the settings file path for configuring ",
            "the sigmadsp backend application",
        ),
    )
    arguments = argument_parser.parse_args()

    settings = SigmadspSettings(arguments.settings)
    launch(settings)


if __name__ == "__main__":
    main()



# -*- coding: utf-8 -*-
from setuptools import setup

packages = \
['sigmadsp',
 'sigmadsp.application',
 'sigmadsp.communication',
 'sigmadsp.generated',
 'sigmadsp.generated.backend_service',
 'sigmadsp.hardware',
 'sigmadsp.helper']

package_data = \
{'': ['*']}

install_requires = \
['PyYAML>=6.0,<7.0',
 'RPi.GPIO>=0.7.1,<0.8.0',
 'grpcio>=1.44.0,<2.0.0',
 'protobuf>=3.20.1,<4.0.0',
 'retry>=0.9.2,<0.10.0',
 'spidev>=3.5,<4.0']

entry_points = \
{'console_scripts': ['sigmadsp = sigmadsp.application.frontend:main',
                     'sigmadsp-backend = sigmadsp.application.backend:main']}

setup_kwargs = {
    'name': 'sigmadsp',
    'version': '1.5.4',
    'description': 'A package for controlling Analog Devices Sigma DSP chipsets.',
    'long_description': '[![pre-commit](https://img.shields.io/badge/pre--commit-enabled-brightgreen?logo=pre-commit&logoColor=white)](https://github.com/pre-commit/pre-commit) [![pre-commit.ci status](https://results.pre-commit.ci/badge/github/elagil/sigmadsp/main.svg)](https://results.pre-commit.ci/latest/github/elagil/sigmadsp/main)\n# Analog Devices Sigma DSP control software\n\nThis software package is a Python application, which controls Analog Devices\ndigital signal processor ([DSP](https://en.wikipedia.org/wiki/Digital_signal_processor)) chipsets. It exposes a TCP server for\nconnecting with SigmaStudio, allowing to upload new applications to the DSP, as well as debugging it. Essentially, it\nbehaves like a wired debug probe, but with an Ethernet connection. This source code was inspired by [the original TCP service](https://wiki.analog.com/resources/tools-software/linux-software/sigmatcp),\nas well as the [hifiberry-dsp](https://github.com/hifiberry/hifiberry-dsp) project.\n\nHowever, this application was written completely from scratch, in an effort to make it more efficient, stable, and faster.\n\nThis software package contains two separate components: a backend service, as well as a frontend interface. It is meant\nto run on single-board computers that connect to an Analog Devices DSP via the serial peripheral interface ([SPI](https://en.wikipedia.org/wiki/Serial_Peripheral_Interface)).\n\n## Backend service\n\nThe backend service is the core application, which\n- connects to the DSP via SPI,\n- exposes a TCP interface towards SigmaStudio,\n- and provides a remote procedure call (RPC) interface, based on [grpc](https://grpc.io/).\n\nWith the latter, a frontend can connect to the backend service and control it remotely.\n\n## Frontend interface\n\nThe frontend interface connects to the RPC service of the backend, allowing the user to control\nsettings via a command-line interface (CLI).\n\n## Supported chipsets\n\nThis is not an extensive list, but only comprises chips that are tested or likely compatible.\n\nDSP|Status|Backend settings `dsp_type`\n---|---|--\n[ADAU145X](https://www.analog.com/media/en/technical-documentation/data-sheets/ADAU1452_1451_1450.pdf) | Fully tested (ADAU1452) | `adau14xx`\n[ADAU146X](https://www.analog.com/media/en/technical-documentation/data-sheets/ADAU1463-1467.pdf) | Untested, register compatible with ADAU145X | `adau14xx`\n\n## Installation\n:zap: **Running the installation will overwrite your existing configuration.** For upgrading, see [Upgrading](#upgrading)!\n\nFor installing, please install git first, then clone this repository and run the installation script.\n\n```bash\nsudo apt install git &&\ngit clone https://github.com/elagil/sigma-dsp.git &&\ncd sigma-dsp &&\n./install.sh\n```\n\nThe script installs the Python package, which includes the `sigmadsp-backend` (the backend) and `sigmadsp` (the frontend) executables.\nIt also sets up a system service, which runs `sigmadsp-backend` in the background.\n\n## Upgrading\n\nFor upgrading, the installation procedure can be repeated, but will overwrite the current configuration file.\n\nInstead, simply upgrade the Python package and restart the backend service:\n\n```bash\nsudo pip3 install sigmadsp --upgrade &&\nsudo systemctl restart sigmadsp-backend.service\n```\n\n\n## Removal\n\nFrom within the previously cloned repository folder `sigma-dsp` run\n\n```bash\n./uninstall.sh\n```\n\n## Configuration\n\nConfiguration of `sigmadsp` is done via a `*.yaml` file, which is created during installation. Its default path is `/var/lib/config.yaml`.\n\n## Usage\n\nFor a list of commands that can be emitted by the frontend, simply type\n\n```bash\nsigmadsp -h\n```\n',
    'author': 'Adrian Figueroa',
    'author_email': 'elagil@takanome.de',
    'maintainer': None,
    'maintainer_email': None,
    'url': 'https://github.com/elagil/sigmadsp',
    'packages': packages,
    'package_data': package_data,
    'install_requires': install_requires,
    'entry_points': entry_points,
    'python_requires': '>=3.7,<4.0',
}


setup(**setup_kwargs)



"""This module contains a class that handles settings for this application."""
import logging
from typing import List, Union

import yaml

from sigmadsp.helper.parser import Parser

# A logger for this module
logger = logging.getLogger(__name__)


class SigmadspSettings:
    """This class holds and manages settings for the SigmaDSP application."""

    default_config_path = "/var/lib/sigmadsp/config.yaml"

    def __init__(self, config_path: str = None):
        """Load a config file in *.yaml format from a specified path.

        If no file is provided, the default path is used for loading settings.

        Args:
            config_path (str, optional): The input path of the settings file.
                Defaults to None.
        """
        if config_path is None:
            config_path = SigmadspSettings.default_config_path

        try:
            # Open settings file, in order to configure the application
            with open(config_path, "r", encoding="utf8") as settings_file:
                self.config = yaml.safe_load(settings_file)
                logger.info("Settings file %s was loaded.", config_path)

        except FileNotFoundError:
            logger.error("Settings file not found at %s. Aborting.", config_path)
            raise

        # A parser for parameters, defined in the config file.
        self.parameter_parser: Union[Parser, None] = None

        self.load_parameters()

    def load_parameters(self) -> None:
        """Load parameter cells, according to the parameter file path that is defined in the settings object."""
        try:
            parser = Parser()
            parser.run(self.config["parameters"]["path"])

            self.parameter_parser = parser

        except (IndexError, TypeError):
            logger.info("No parameter path was defined in the configuration file.")

    def store_parameters(self, lines: List[str]):
        """Store parameters to the parameter file.

        Args:
            lines (List[str]): [description]
        """
        with open(self.config["parameters"]["path"], "w", encoding="UTF8") as parameter_file:
            parameter_file.writelines(lines)

        self.load_parameters()
